// Enemy.cpp: implementation of the Enemy class.
//
//////////////////////////////////////////////////////////////////////

#include "Enemy.h"
#include <stdlib.h>

const int DIR_NONE = 0;
const int DIR_RIGHT = 1;
const int DIR_LEFT = 2;
const int DIR_UP = 3;
const int DIR_DOWN = 4;

const int BLOCK_WIDTH = 32;
const int BLOCK_HEIGHT = 32;

const int SPRITE_WIDTH = 32;
const int SPRITE_HEIGHT = 32;

const int STEP_X = 2;
const int STEP_Y = 2;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

Enemy::Enemy(SDL_Surface* frames, MapInfo* info, Hero* heroSprite) : Sprite(frames)
{
	int iBlockX;
	int iBlockY;

	int iRetry = 500;

	int heroBlockX;
	int heroBlockY;

	int heroDistance = 0;

	int mapBlocksX;
	int mapBlocksY;

	InitFrames();
	mapInfo = info;

	// Chose a starting location far from the player.
	heroBlockX = (heroSprite->getX() - (heroSprite->getX() % BLOCK_WIDTH)) / BLOCK_WIDTH;
	heroBlockY = (heroSprite->getY() - (heroSprite->getY() % BLOCK_HEIGHT)) / BLOCK_HEIGHT;

	mapBlocksX = mapInfo->getWidth() / BLOCK_WIDTH;
	mapBlocksY = mapInfo->getHeight() / BLOCK_HEIGHT;

	iBlockX = rand() % mapBlocksX;
	iBlockY = rand() % mapBlocksY;

	heroDistance = ((heroBlockX - iBlockX) * (heroBlockX - iBlockX)) + 
		           ((heroBlockY - iBlockY) * (heroBlockY - iBlockY));

	// if cant find one in 500 tries, just find a blank one.
	while ((heroDistance < 16) || (mapInfo->getBlockValue(iBlockX * BLOCK_WIDTH, iBlockY * BLOCK_HEIGHT) != 0))
	{
		iBlockX = rand() % mapBlocksX;
		iBlockY = rand() % mapBlocksY;

		heroDistance = ((heroBlockX - iBlockX) * (heroBlockX - iBlockX)) + 
					   ((heroBlockY - iBlockY) * (heroBlockY - iBlockY));

		if (iRetry > 0)
		{
			iRetry--;
		}
		else
		{
			//Retried too many times
			//Forget the distance check and set it to 100.
			heroDistance = 100;
		}
	}

	setX(iBlockX * BLOCK_WIDTH);
	setY(iBlockY * BLOCK_HEIGHT);
}

Enemy::~Enemy()
{
	mapInfo = NULL;
}

void Enemy::InitFrames()
{
	const int FRAME_COUNT = 4;

	if (frameRect == NULL)
	{
		delete[] frameRect;
		frameRect = NULL;
	}

	frameRect = new SDL_Rect[FRAME_COUNT];

	for (int i = 0;i< FRAME_COUNT;++i)
	{
		frameRect[i].x = i * SPRITE_WIDTH;
		frameRect[i].y = 0;
		frameRect[i].w = SPRITE_WIDTH;
		frameRect[i].h = SPRITE_HEIGHT;
	}
	iFrame = 0;
}

int Enemy::getWidth()
{
	return SPRITE_WIDTH;
}

int Enemy::getHeight()
{
	return SPRITE_HEIGHT;
}

void Enemy::update()
{

	int upDist = 0;
	int downDist = 0;
	int leftDist = 0;
	int rightDist = 0;

	if (((getX() % BLOCK_WIDTH) == 0) && ((getY() % BLOCK_HEIGHT) == 0))
	{
		//Switch directions at a block boundary.
		int availableDir[4] = {-1, -1, -1, -1};

		int availableCount = 0;

		// May I move up.
		upDist = mapInfo->howFarUp(getX(), getY(), STEP_Y);
		downDist = mapInfo->howFarDown(getX(), getY(), STEP_Y);

		//How about Left, Right
		leftDist = mapInfo->howFarLeft(getX(), getY(), STEP_X);
		rightDist = mapInfo->howFarRight(getX(), getY(), STEP_Y);


		if (upDist > 0)
		{
			availableDir[availableCount++] = DIR_UP;
		}

		if (downDist > 0)
		{
			availableDir[availableCount++] = DIR_DOWN;
		}

		if (leftDist > 0)
		{
			availableDir[availableCount++] = DIR_LEFT;
		}

		if (rightDist > 0)
		{
			availableDir[availableCount++] = DIR_RIGHT;
		}

		if (availableCount == 0)
		{
			iDir = DIR_NONE;
		}
		else
		{
			iDir = availableDir[rand() % availableCount];
		}
	}

	switch (iDir)
	{
	case DIR_UP:
		upDist = mapInfo->howFarUp(getX(), getY(), STEP_Y);
		setY(getY() - upDist);
		iFrame = 3;
		break;
	case DIR_DOWN:
		downDist = mapInfo->howFarDown(getX(), getY(), STEP_Y);
		setY(getY() + downDist);
		iFrame = 2;
		break;
	case DIR_LEFT:
		leftDist = mapInfo->howFarLeft(getX(), getY(), STEP_X);
		setX(getX() - leftDist);
		iFrame = 1;
		break;
	case DIR_RIGHT:
		rightDist = mapInfo->howFarRight(getX(), getY(), STEP_X);
		setX(getX() + rightDist);
		iFrame = 0;
		break;
	}

}
