// Hero.cpp: implementation of the Hero class.
//
//////////////////////////////////////////////////////////////////////

#include "Hero.h"

const int DIR_NONE = 0;
const int DIR_RIGHT = 1;
const int DIR_LEFT = 2;
const int DIR_UP = 3;
const int DIR_DOWN = 4;

const int BLOCK_WIDTH = 32;
const int BLOCK_HEIGHT = 32;

const int SPRITE_WIDTH = 32;
const int SPRITE_HEIGHT = 32;

const int STEP_X = 4;
const int STEP_Y = 4;

const int FRAME_WAIT_MAX = 4;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

Hero::Hero(SDL_Surface* frames, MapInfo* info) : Sprite(frames)
{
	keyDown = false;
	InitFrames();
	iFrameWait = FRAME_WAIT_MAX;
	iDir = DIR_NONE;
	iNextDir = DIR_NONE;
	mapInfo = info;
}

Hero::~Hero()
{
	mapInfo = NULL;
}

int Hero::getWidth()
{
	return SPRITE_WIDTH;
}

int Hero::getHeight()
{
	return SPRITE_HEIGHT;
}

void Hero::InitFrames()
{
	const int FRAME_COUNT = 8;

	if (frameRect == NULL)
	{
		delete[] frameRect;
		frameRect = NULL;
	}

	frameRect = new SDL_Rect[FRAME_COUNT];

	for (int i = 0;i< FRAME_COUNT;++i)
	{
		frameRect[i].x = i * SPRITE_WIDTH;
		frameRect[i].y = 0;
		frameRect[i].w = SPRITE_WIDTH;
		frameRect[i].h = SPRITE_HEIGHT;
	}
	iFrame = 0;
}

void Hero::update()
{
	bool switchDir = false;

	int iStep = 0;

	iFrameWait--;
	if (iFrameWait <= 0)
	{
		switch(iDir)
		{
		case DIR_LEFT:
			if (iFrame == 2) 
				iFrame = 3;
			else
				iFrame = 2;
			break;
		case DIR_RIGHT:
			if (iFrame == 0)
				iFrame = 1;
			else
				iFrame = 0;
			break;
		case DIR_UP:
			if (iFrame == 6)
				iFrame = 7;
			else
				iFrame = 6;
			break;
		case DIR_DOWN:
			if (iFrame == 4)
				iFrame = 5;
			else
				iFrame = 4;
			break;
		}
		iFrameWait = FRAME_WAIT_MAX;
	}

	// Update position
	switchDir = false;
	switch(iDir)
	{
	case DIR_UP:
		iStep = mapInfo->howFarUp(getX(), getY(), STEP_Y);
		setY(getY() - iStep);
		switchDir = ((getY() % BLOCK_HEIGHT) == 0);
		break;
	case DIR_DOWN:
		iStep = mapInfo->howFarDown(getX(), getY(), STEP_Y);
		setY(getY() + iStep);
		switchDir = ((getY() % BLOCK_HEIGHT) == 0);
		break;
	case DIR_LEFT:
		iStep = mapInfo->howFarLeft(getX(), getY(), STEP_X);
		setX(getX() - iStep);
		switchDir = ((getX() % BLOCK_WIDTH) == 0);
		break;
	case DIR_RIGHT:
		iStep = mapInfo->howFarRight(getX(), getY(), STEP_X);
		setX(getX() + iStep);
		switchDir = ((getX() % BLOCK_WIDTH) == 0);
		break;
	}

	if (iStep == 0)
	{
		iDir = iNextDir;
/*
		if (!keyDown)
		{
			iDir = iNextDir; // was DIR_NONE
			iNextDir = DIR_NONE;
		}
		else
		{
			iDir = iNextDir;
		}
*/
	}
	else if (switchDir)
	{
		iDir = iNextDir;
		if (!keyDown)
		{
			iNextDir = DIR_NONE;
		}
		else
		{
			iNextDir = iNextDir;
		}
		iFrameWait = 0;
	}
}

void Hero::moveLeft()
{
	if (iDir == DIR_NONE)
	{
		iDir = DIR_LEFT;
		iNextDir = iDir;
		iFrameWait = 0;
	}
	else
	{
		iNextDir = DIR_LEFT;
	}
	setKeyDown(true);
}

void Hero::moveRight()
{
	if (iDir == DIR_NONE)
	{
		iDir = DIR_RIGHT;
		iNextDir = iDir;
		iFrameWait = 0;
	}
	else
	{
		iNextDir = DIR_RIGHT;
	}
	setKeyDown(true);
}

void Hero::moveUp()
{
	if (iDir == DIR_NONE)
	{
		iDir = DIR_UP;
		iNextDir = iDir;
		iFrameWait = 0;
	}
	else
	{
		iNextDir = DIR_UP;
	}
	setKeyDown(true);

}

void Hero::moveDown()
{
	if (iDir == DIR_NONE)
	{
		iDir = DIR_DOWN;
		iNextDir = iDir;
		iFrameWait = 0;
	}
	else
	{
		iNextDir = DIR_DOWN;
	}
	setKeyDown(true);
}

void Hero::setKeyDown(bool value)
{
	keyDown = value;
	if (value == false) 
		iNextDir = DIR_NONE;
}
