// MapInfo.cpp: implementation of the MapInfo class.
//
//////////////////////////////////////////////////////////////////////

#include "MapInfo.h"

const int BLOCK_WIDTH = 32;
const int BLOCK_HEIGHT = 32;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

MapInfo::MapInfo(int* data, int width, int height)
{
	mapData = data;
	mapWidth = width;
	mapHeight = height;
}

MapInfo::~MapInfo()
{

}

int MapInfo::getWidth()
{
	return mapWidth * BLOCK_WIDTH;
}

int MapInfo::getHeight()
{
	return mapHeight * BLOCK_HEIGHT;
}

int MapInfo::howFarUp(int x, int y, int desiredStep)
{
	int blockX;
	int blockY;
	int desiredBlockValue;
	int currentBlockValue;

	int stepRet = 0;

	if (desiredStep > BLOCK_HEIGHT)
		stepRet = BLOCK_HEIGHT;
	else
		stepRet = desiredStep;

	if ((y % BLOCK_HEIGHT) < stepRet)
	{
		
		blockX = (x - (x % BLOCK_WIDTH)) / BLOCK_WIDTH;
		blockY = (y - (y % BLOCK_HEIGHT)) / BLOCK_HEIGHT;

		if (blockX < 0) 
			blockX = 0;
		else if (blockX >= mapWidth)
			blockX = mapWidth - 1;

		if (blockY < 0)
			blockY = 0;
		else if (blockY >= mapHeight)
			blockY = mapHeight - 1;

		currentBlockValue = mapData[(blockY * mapWidth) + blockX];

		if (currentBlockValue == 0)
		{
			if (y - stepRet < 0)
			{
				stepRet = y;
			}

			blockX = (x - (x % BLOCK_WIDTH)) / BLOCK_WIDTH;
			blockY = ((y - stepRet) - ((y - stepRet) % BLOCK_HEIGHT)) / BLOCK_HEIGHT;

			if (blockX < 0) 
				blockX = 0;
			else if (blockX >= mapWidth)
				blockX = mapWidth - 1;

			if (blockY < 0)
				blockY = 0;
			else if (blockY >= mapHeight)
				blockY = mapHeight - 1;

			desiredBlockValue = mapData[(blockY * mapWidth) + blockX];

			if (desiredBlockValue != 0)
			{
				// Limit to top of block.  Next block up is blocked.
				stepRet = (y % BLOCK_HEIGHT);
			}
		}
		else
		{
			stepRet = 0; // Can't move anywhere if inside a block.
		}
	}

	return stepRet;
}

int MapInfo::howFarDown(int x, int y, int desiredStep)
{
	int blockX;
	int blockY;
	int desiredBlockValue;
	int currentBlockValue;

	int stepRet = 0;

	if (desiredStep > BLOCK_HEIGHT)
		stepRet = BLOCK_HEIGHT;
	else
		stepRet = desiredStep;

	//if ((BLOCK_HEIGHT - (y % BLOCK_HEIGHT)) < stepRet)
	//if ((y % BLOCK_HEIGHT) < stepRet)
	{
		
		blockX = (x - (x % BLOCK_WIDTH)) / BLOCK_WIDTH;
		blockY = (y - (y % BLOCK_HEIGHT)) / BLOCK_HEIGHT;

		if (blockX < 0) 
			blockX = 0;
		else if (blockX >= mapWidth)
			blockX = mapWidth - 1;

		if (blockY < 0)
			blockY = 0;
		else if (blockY >= mapHeight)
			blockY = mapHeight - 1;

		currentBlockValue = mapData[(blockY * mapWidth) + blockX];

		if (currentBlockValue == 0)
		{
			if (y + stepRet >= ((mapHeight - 1) * BLOCK_HEIGHT))
			{
				stepRet = ((mapHeight - 1) * BLOCK_HEIGHT) - y;
			}

			blockX = (x - (x % BLOCK_WIDTH)) / BLOCK_WIDTH;
			blockY = (y + stepRet + (BLOCK_HEIGHT - ((y + stepRet + 1) % BLOCK_HEIGHT))) / BLOCK_HEIGHT;
			//blockY = (y + stepRet - ((y + stepRet) % BLOCK_HEIGHT)) / BLOCK_HEIGHT;

			if (((y + stepRet) % BLOCK_HEIGHT) > 0)
			{
				blockY++;
			}

			if (blockX < 0) 
				blockX = 0;
			else if (blockX >= mapWidth)
				blockX = mapWidth - 1;

			if (blockY < 0)
				blockY = 0;
			else if (blockY >= mapHeight)
				blockY = mapHeight - 1;

			desiredBlockValue = mapData[(blockY * mapWidth) + blockX];

			if (desiredBlockValue != 0)
			{
				// Limit to top of block.  Next block down is blocked.
				stepRet = (BLOCK_HEIGHT - (y % BLOCK_HEIGHT));
				if (stepRet == BLOCK_HEIGHT)
				{
					stepRet = 0;
				}
			}
		}
		else
		{
			stepRet = 0; // Can't move anywhere if inside a block.
		}
	}

	return stepRet;
}

int MapInfo::howFarLeft(int x, int y, int desiredStep)
{
	int blockX;
	int blockY;
	int desiredBlockValue;
	int currentBlockValue;

	int stepRet = 0;

	if (desiredStep > BLOCK_WIDTH)
		stepRet = BLOCK_WIDTH;
	else
		stepRet = desiredStep;

	if ((x % BLOCK_WIDTH) < stepRet)
	{
		
		blockX = (x - (x % BLOCK_WIDTH)) / BLOCK_WIDTH;
		blockY = (y - (y % BLOCK_HEIGHT)) / BLOCK_HEIGHT;

		if (blockX < 0) 
			blockX = 0;
		else if (blockX >= mapWidth)
			blockX = mapWidth - 1;

		if (blockY < 0)
			blockY = 0;
		else if (blockY >= mapHeight)
			blockY = mapHeight - 1;

		currentBlockValue = mapData[(blockY * mapWidth) + blockX];

		if (currentBlockValue == 0)
		{
			if (x - stepRet < 0)
			{
				stepRet = x;
			}

			blockX = ((x - stepRet) - ((x - stepRet) % BLOCK_WIDTH)) / BLOCK_WIDTH;
			blockY = (y - (y % BLOCK_HEIGHT)) / BLOCK_HEIGHT;

			if (blockX < 0) 
				blockX = 0;
			else if (blockX >= mapWidth)
				blockX = mapWidth - 1;

			if (blockY < 0)
				blockY = 0;
			else if (blockY >= mapHeight)
				blockY = mapHeight - 1;

			desiredBlockValue = mapData[(blockY * mapWidth) + blockX];

			if (desiredBlockValue != 0)
			{
				// Limit to left of block.  Next block left is blocked.
				stepRet = (x % BLOCK_WIDTH);
			}
		}
		else
		{
			stepRet = 0; // Can't move anywhere if inside a block.
		}
	}

	return stepRet;
}

int MapInfo::howFarRight(int x, int y, int desiredStep)
{
	int blockX;
	int blockY;
	int desiredBlockValue;
	int currentBlockValue;

	int stepRet = 0;

	if (desiredStep > BLOCK_WIDTH)
		stepRet = BLOCK_WIDTH;
	else
		stepRet = desiredStep;

	//if ((BLOCK_WIDTH - (x % BLOCK_WIDTH)) < stepRet)
	if ((x % BLOCK_WIDTH) < stepRet)
	{
		
		blockX = (x - (x % BLOCK_WIDTH)) / BLOCK_WIDTH;
		blockY = (y - (y % BLOCK_WIDTH)) / BLOCK_WIDTH;

		if (blockX < 0) 
			blockX = 0;
		else if (blockX >= mapWidth)
			blockX = mapWidth - 1;

		if (blockY < 0)
			blockY = 0;
		else if (blockY >= mapHeight)
			blockY = mapHeight - 1;

		currentBlockValue = mapData[(blockY * mapWidth) + blockX];

		if (currentBlockValue == 0)
		{
			if (x + stepRet >= ((mapWidth - 1) * BLOCK_WIDTH))
			{
				stepRet = ((mapWidth - 1) * BLOCK_WIDTH) - x;
			}

			blockX = ((x + stepRet) - ((x + stepRet) % BLOCK_WIDTH)) / BLOCK_WIDTH;
			if (((x + stepRet) % BLOCK_WIDTH) > 0)
			{
				blockX++;
			}
			blockY = (y - (y % BLOCK_WIDTH)) / BLOCK_WIDTH;

			if (blockX < 0) 
				blockX = 0;
			else if (blockX >= mapWidth)
				blockX = mapWidth - 1;

			if (blockY < 0)
				blockY = 0;
			else if (blockY >= mapHeight)
				blockY = mapHeight - 1;

			desiredBlockValue = mapData[(blockY * mapWidth) + blockX];

			if (desiredBlockValue != 0)
			{
				// Limit to left of block.  Next right up is blocked.
				stepRet = (x % BLOCK_WIDTH); // (BLOCK_WIDTH - (y % BLOCK_WIDTH));
				if (stepRet == BLOCK_WIDTH)
				{
					stepRet = 0;
				}
			}
		}
		else
		{
			stepRet = 0; // Can't move anywhere if inside a block.
		}
	}
	
	return stepRet;
}

int MapInfo::getBlockValue(int x, int y)
{
	int iBlockX;
	int iBlockY;

	iBlockX = (x - (x % BLOCK_WIDTH)) / BLOCK_WIDTH;
	iBlockY = (y - (y % BLOCK_HEIGHT)) / BLOCK_HEIGHT;
	return mapData[(iBlockY * mapWidth) + iBlockX];
}
